/*
 * 06_Ultra_Sonic.cpp
 * Created: 2020
 *
 * Pindescription Arduino Uno
 * No.	Func.	PIN-MC	Used as In/Out Res. Pindefinition
 * 0	RX		PD0		RX		In
 * 1	TX		PD1		TX		In
 * 2	INT0	PD2		US-Echo	In
 * 3	Digit	PD3		Servo	Out
 * 4	Digit	PD4		Line-M	In
 * 5	OC0B	PD5		PWM-A	Out
 * 6	OC0B	PD6		PWM-B	Out
 * 7	PCINT23	PD7		Color-1	In	->	DDRD = 0b01101000
 * 8	Digit	PB0		Motor-A	Out
 * 9	Digit	PB1		Motor-B	Out
 *10	Digit	PB2		Line-L	In
 *11	PCINT3	PB3		Color-2	In
 *12	Digit	PB4		Infr.	In	Not needed
 *13	LED		PB5		LED		Out	->	DDRB = 0b00100011
 *A0	Ana		PC0		Line-RA	In
 *A1	Ana		PC1		Line-LA	In
 *A2	Ana		PC2		Dist-1	In
 *A3	Ana		PC3		Dist-2	In
 *A4	Digit	PC4		Line-R	In
 *A5	Digit	PC5		US-Trig	Out	->	DDRC = 0b00100000
 */ 
#define F_CPU 16000000UL //16MHz required for delay
#include <avr/io.h> //Input/Output library
#include <util/delay.h> //Needed for using _delay_...
#include <avr/interrupt.h> //External and internal Interrupts
#include "Init.h"
#include "USART_Functions.h"
#include "Interrupt_Service_Routines.h"

//Definitions
#define FOSC				16000000 // Clock Speed
#define BAUD				76800//Baud rate, normally used by Arduino 9600, possible rates: n * 9600
#define MYUBRR				FOSC/16/BAUD-1 //Calculate my baud rate

//Macros
#define	LED_ON				PORTB |= (1 << PORTB5) //Push PORTB pin 5 high (LED, pin 13 "Uno")
#define	LED_OFF				PORTB &= ~(1 << PORTB5) //~ Not Operator
#define	LED_TOGGLE			PINB |= (1<<PINB5) //Toggle Arduino Uno pin 13


//Variables
//unsigned char z; //For tests

//unsigned int T2ck; //Counter TMR2 overflow
//unsigned char Line_digital_right; //State of right line sensor
//unsigned char Line_digital_middle; //State of middle line sensor
//unsigned char Line_digital_left; //State of left line sensor

//unsigned char US_Time; //Runtime of US signal, HI part
//unsigned char Counter_US; //Counter TMR2 overflow generating a US-trigger every 32ms

//unsigned char Data[10]; //Number of datas displayed on Data Visualizer

//Funktions, declaration
void Init (void); //Initialize Microcontroller
void USART_Transmit(unsigned char); //Transmit ASCII to Data Visualizer
void USART_Init(unsigned int); //Initialize USART Transmission/Reception
void Transmit_literal (char); //Transmit literal to Data Visualizer Monitor
void Data_Visualizer (void); //Define datas to be vizualized


//Main routine
int main(void) {
	Init(); //Initalize the MC
	USART_Init(MYUBRR); //Transmision/reception initialize
	sei(); //Enable all interrupts
	//cli(); //Clear all interrupts
	while (1) {
		Data_Visualizer(); //Define the values to be displayed and transmit them
		if (US_Time > 10) {
		LED_ON;
		}
		else LED_OFF;
	}
}

/***************************************************************************/
//Interrupt service routines (ISR)



/***********************************************************************************************/
//Other functions

//Data beeing transmitted to the Data Visualizer
void Data_Visualizer (void) {
	Data[0] = Line_digital_right; //First value at the console
	Data[1] = Line_digital_middle; //
	Data[2] = Line_digital_left; //
	Data[3] = US_Time; //Runtime of the US-signal, HI-part of TMR1-value
	for (unsigned char i = 0; i < 4; i++) {
		Transmit_literal(Data[i]); //Three digit literal unsigned
	}
	USART_Transmit('\n'); //New line
	USART_Transmit('\r'); //Cariage return
}



/***************************************************************************************/




